// use next line if working with MPLAB IDE
#define ICD2_DEBUG

// use next 2 lines if NOT working with MPLAB IDE (using compila.bat)
//#include "16F628A.H"
//#pragma cdata[0x2007] = 0x3F34 // configuration word for this application

#include "int16cxx.h"
#define BYTE uns8
#define WORD uns16

// FW Version: 8.112 (y.mdd)

// Variables
BYTE vidram[60]; // video ram for 10 characters
BYTE col; // columns counter for one character
BYTE dac; // dac output value register
BYTE mask; // mask for column dots
BYTE fase; // machine state
BYTE scrollix; // index for char scrolling
BYTE numchars; // num of chars in scrolling text
BYTE framecnt; // frames counter for scrolling speed
BYTE speed; // scrolling speed (read from eeprom)
BYTE mc; // generic BYTE variable used in main

// Constants

const BYTE chrgen[] = {
 0,0,0,0,0,				// chars 0x20-0x2f
 0,0,95,0,0,
 0,3,0,3,0,
 20,62,20,62,20,
 36,42,107,42,18,
 68,32,16,8,68,
 54,73,85,34,80,
 0,3,1,0,0,
 0,28,34,65,0,
 0,65,34,28,0,
 42,28,62,28,42,
 8,8,62,8,8,
 0,0,96,0,0,
 8,8,8,8,8,
 0,0,64,0,0,
 64,32,16,8,4,

 62,81,73,69,62,            // chars 0x30-0x3f
 0,66,127,64,0,
 66,97,81,73,70,
 34,65,73,73,54,
 24,20,18,127,16,
 71,69,69,69,57,
 60,74,73,73,48,
 97,17,9,5,3,
 54,73,73,73,54,
 6,73,73,41,30,
 0,0,36,0,0,
 0,0,100,0,0,
 0,8,20,34,65,
 0,20,20,20,0,
 65,34,20,8,0,
 2,1,81,9,6,

 62,65,77,77,14,            // chars 0x40-0x4f
 126,9,9,9,126,
 127,73,73,73,54,
 62,65,65,65,34,
 127,65,65,34,28,
 127,73,73,73,65,
 127,9,9,9,1,
 62,65,65,73,58,
 127,8,8,8,127,
 0,65,127,65,0,
 32,64,65,63,1,
 127,8,20,34,65,
 127,64,64,64,64,
 127,2,4,2,127,
 127,6,8,48,127,
 62,65,65,65,62,

 127,9,9,9,6,           // chars 0x50-0x5f
 62,65,65,97,126,
 127,9,25,41,70,
 70,73,73,73,49,
 1,1,127,1,1,
 63,64,64,64,63,
 31,32,64,32,31,
 127,32,16,32,127,
 99,20,8,20,99,
 3,4,120,4,3,
 97,81,73,69,67,
 0,127,65,65,0,
 4,8,16,32,64,
 0,65,65,127,0,
 4,2,1,2,4,
 64,64,64,64,64 };

void Earom2Bitmap(void)
{
BYTE c, x, tmp, start;
WORD i;

    start = scrollix; // set starting point for eeprom read
    for(x = 0; x < 60; ) // fill vidram
        {
        EEADR = start; // set earom address
        EECON1.0 = 1; // read command
        c = EEDATA; // get earom data
        c -= 0x20; // character map starts from 0x20
        c &= 0x3F; // limit the range to actual character generator 
        i = (WORD)c * 5;
        for(c = 0; c < 5; ++c)
            {
            tmp = chrgen[i++];
            vidram[x++] = tmp;
            }
        vidram[x++] = 0; // space between characters    
        if(++start >= numchars) // next earom address; if end of string
            start = 0; // reset pointer to the first eeprom addr
        }
    if(++framecnt >= speed) // if n cycles with same pattern
        {
        framecnt = 0; // reset frame counter
        if(++scrollix >= numchars) // if end of text reached
            scrollix = 0; // reset scrolling index
        }

}

// Main procedure
// --------------
void main(void)
{
    OPTION_REG = 0x01; // 0000.0001 set options (prescaler to tmr0, rate 1:4)
 
    PORTA = 0x00; // 0000.0000 set portA data
    TRISA = 0x24; // 0010.0100 set portA direction (0=out)

    PORTB = 0x00; // 0000.0000 set portB data
    TRISB = 0x00; // 0000.0000 set portB direction (0=out)

    VRCON = 0xCF; // 1100.1111 Vref ON, Output on RA2, hi-range, level 15
    CMCON = 0x07; // 0000.0111 comparator module OFF

    clearRAM(); // clear all RAM: compiler internal command

    for(mc = 0; mc < 127; ++mc) // get num of chars in eeprom
        {
        EEADR = mc; // set earom address
        EECON1.0 = 1; // read command
        if(EEDATA == 0xFF) // if end of char string
            break;
        }
    numchars = mc; // set num of chars

    EEADR = 0x7F; // set earom address 0x7F (scroll speed)
    EECON1.0 = 1; // read command
    speed = EEDATA; // read programmed speed

    for(;;)
        {
        clrwdt(); // reset watchdog
        while(T0IF == 0); // wait for tmr0 overflow
        T0IF = 0; // clear timer0 overflow flag
        switch(fase)
            {
            case 0: // out low level for trigger
                VRCON = 0xE0; // out Min level on RA2
                ++fase; // next machine state
                break;
            case 1: // output characters (10*6 columns)
                for(col = 0; col < 60; ++col) // for all the columns
                    {
                    clrwdt(); // reset watchdog
                    for(dac = 0xE0; dac < 0xE9; ++dac); // delay and set dac at first dot level
                    for(mask = 0x40; mask > 0; mask >>= 1) // for all the rows
                        {
                        if(vidram[col] & mask) // if dot is ON
                            VRCON = dac; // out right level on RA2
                        else // if dot is OFF
                            VRCON = 0xE0; // out Min level on RA2
                        ++dac; // next dac value
                        }
                    VRCON = 0xE0; // out Min level on RA2
                    }
                TMR0 = 256-40; // next overflow in 40 ticks
                T0IF = 0; // clear timer0 overflow flag
                ++fase; // next machine state
                break;
            case 2: // terminate scan line
                VRCON = 0xCF; // output high level
                T0IF = 1; // set timer0 overflow flag
                Earom2Bitmap(); // reload buffer and scroll text
                fase = 0; // next machine state
                break;
            }
        }
}
